@@ -16,7 +16,7 @@
 
 # A set of helpers for starting/running etcd for tests
 
-ETCD_VERSION=${ETCD_VERSION:-3.5.10}
+ETCD_VERSION=${ETCD_VERSION:-3.5.7}
 ETCD_HOST=${ETCD_HOST:-127.0.0.1}
 ETCD_PORT=${ETCD_PORT:-2379}
 # This is intentionally not called ETCD_LOG_LEVEL:
@@ -51,9 +51,7 @@ kube::etcd::validate() {
 
   # need set the env of "ETCD_UNSUPPORTED_ARCH" on unstable arch.
   arch=$(uname -m)
-  if [[ $arch =~ aarch* ]]; then
-	  export ETCD_UNSUPPORTED_ARCH=arm64
-  elif [[ $arch =~ arm* ]]; then
+  if [[ $arch =~ arm* ]]; then
 	  export ETCD_UNSUPPORTED_ARCH=arm
   fi
   # validate installed version is at least equal to minimum
@@ -162,7 +160,7 @@ kube::etcd::install() {
 
     if [[ ${os} == "darwin" ]]; then
       download_file="etcd-v${ETCD_VERSION}-${os}-${arch}.zip"
-      url="https://github.com/etcd-io/etcd/releases/download/v${ETCD_VERSION}/${download_file}"
+      url="https://github.com/coreos/etcd/releases/download/v${ETCD_VERSION}/${download_file}"
       kube::util::download_file "${url}" "${download_file}"
       unzip -o "${download_file}"
       ln -fns "etcd-v${ETCD_VERSION}-${os}-${arch}" etcd
