@@ -24,6 +24,7 @@ KUBE_VERBOSE="${KUBE_VERBOSE:-1}"
 
 KUBE_ROOT=$(dirname "${BASH_SOURCE[0]}")/..
 source "${KUBE_ROOT}/hack/lib/init.sh"
+source "${KUBE_ROOT}/hack/lib/protoc.sh"
 cd "${KUBE_ROOT}"
 
 kube::golang::setup_env
@@ -33,74 +34,55 @@ GENERATED_FILE_PREFIX="${GENERATED_FILE_PREFIX:-zz_generated.}"
 UPDATE_API_KNOWN_VIOLATIONS="${UPDATE_API_KNOWN_VIOLATIONS:-}"
 
 OUT_DIR="_output"
-BIN_DIR="${OUT_DIR}/bin"
 PRJ_SRC_PATH="k8s.io/kubernetes"
 BOILERPLATE_FILENAME="vendor/k8s.io/code-generator/hack/boilerplate.go.txt"
 APPLYCONFIG_PKG="k8s.io/client-go/applyconfigurations"
 
+# Any time we call sort, we want it in the same locale.
+export LC_ALL="C"
+
+# Work around for older grep tools which might have options we don't want.
+unset GREP_OPTIONS
+
 if [[ "${DBG_CODEGEN}" == 1 ]]; then
     kube::log::status "DBG: starting generated_files"
 fi
 
-# This is a partial 'find' command.  The caller is expected to pass the
-# remaining arguments.
-#
-# Example:
-#   kfind -type f -name foobar.go
-function kfind() {
-    # We want to include the "special" vendor directories which are actually
-    # part of the Kubernetes source tree (./staging/*) but we need them to be
-    # named as their ./vendor/* equivalents.  Also, we do not want all of
-    # ./vendor nor ./hack/tools/vendor nor even all of ./vendor/k8s.io.
-    find -H .                      \
-        \(                         \
-        -not \(                    \
-            \(                     \
-                -name '_*' -o      \
-                -name '.[^.]*' -o  \
-                \(                 \
-                  -name 'vendor'   \
-                  -type d          \
-                \) -o              \
-                \(                 \
-                  -name 'testdata' \
-                  -type d          \
-                \)                 \
-            \) -prune              \
-        \)                         \
-        \)                         \
-        "$@"                       \
-        | sed 's|^./staging/src|vendor|'
+function git_find() {
+    # Similar to find but faster and easier to understand.  We want to include
+    # modified and untracked files because this might be running against code
+    # which is not tracked by git yet.
+    git ls-files -cmo --exclude-standard ':!:vendor/*' "$@"
 }
 
-function find_all_go_dirs() {
-    kfind -type f -name \*.go  \
-        | sed 's|/[^/]*$||'    \
-        | sed 's|^./||'        \
-        | LC_ALL=C sort -u
+function git_grep() {
+    # We want to include modified and untracked files because this might be
+    # running against code which is not tracked by git yet.
+    # We need vendor exclusion added at the end since it has to be part of
+    # the pathspecs which are specified last.
+    git grep --untracked "$@" ':!:vendor/*'
 }
 
-# This variable holds a list of every directory that contains Go files in this
-# project.  Other rules and variables can use this as a starting point to
-# reduce filesystem accesses.
-if [[ "${DBG_CODEGEN}" == 1 ]]; then
-    kube::log::status "DBG: finding all *.go dirs"
-fi
-ALL_GO_DIRS=()
-kube::util::read-array ALL_GO_DIRS < <(find_all_go_dirs)
-if [[ "${DBG_CODEGEN}" == 1 ]]; then
-    kube::log::status "DBG: found ${#ALL_GO_DIRS[@]} *.go dirs"
-fi
-
 # Generate a list of all files that have a `+k8s:` comment-tag.  This will be
 # used to derive lists of files/dirs for generation tools.
+#
+# We want to include the "special" vendor directories which are actually part
+# of the Kubernetes source tree (staging/*) but we need them to be named as
+# their vendor/* equivalents.  We do not want all of vendor nor
+# hack/tools/vendor nor even all of vendor/k8s.io - just the subset that lives
+# in staging.
 if [[ "${DBG_CODEGEN}" == 1 ]]; then
     kube::log::status "DBG: finding all +k8s: tags"
 fi
 ALL_K8S_TAG_FILES=()
 kube::util::read-array ALL_K8S_TAG_FILES < <(
-    find "${ALL_GO_DIRS[@]}" -maxdepth 1 -type f -name \*.go -print0 \
-        | xargs -0 grep --color=never -l '^// *+k8s:')
+    git_grep -l \
+        -e '^// *+k8s:'                `# match +k8s: tags` \
+        -- \
+        ':!:*/testdata/*'              `# not under any testdata` \
+        ':(glob)**/*.go'               `# in any *.go file` \
+        | sed 's|^staging/src|vendor|' `# see comments above` \
+    )
 if [[ "${DBG_CODEGEN}" == 1 ]]; then
     kube::log::status "DBG: found ${#ALL_K8S_TAG_FILES[@]} +k8s: tagged files"
 fi
@@ -109,6 +91,120 @@ fi
 # Code generation logic.
 #
 
+# protobuf generation
+#
+# Some of the later codegens depend on the results of this, so it needs to come
+# first in the case of regenerating everything.
+function codegen::protobuf() {
+    # NOTE: All output from this script needs to be copied back to the calling
+    # source tree.  This is managed in kube::build::copy_output in build/common.sh.
+    # If the output set is changed update that function.
+
+    local apis=()
+    kube::util::read-array apis < <(
+        git grep --untracked --null -l \
+            -e '// +k8s:protobuf-gen=package' \
+            -- \
+            cmd pkg staging \
+            | xargs -0 -n1 dirname \
+            | sed 's|^|k8s.io/kubernetes/|;s|k8s.io/kubernetes/staging/src/||' \
+            | sort -u)
+
+    kube::log::status "Generating protobufs for ${#apis[@]} targets"
+    if [[ "${DBG_CODEGEN}" == 1 ]]; then
+        kube::log::status "DBG: generating protobufs for:"
+        for dir in "${apis[@]}"; do
+            kube::log::status "DBG:     $dir"
+        done
+    fi
+
+    git_find -z \
+        ':(glob)**/generated.proto' \
+        ':(glob)**/generated.pb.go' \
+        | xargs -0 rm -f
+
+    if kube::protoc::check_protoc >/dev/null; then
+      hack/update-generated-protobuf-dockerized.sh "${apis[@]}"
+    else
+      kube::log::status "protoc ${PROTOC_VERSION} not found (can install with hack/install-protoc.sh); generating containerized..."
+      build/run.sh hack/update-generated-protobuf-dockerized.sh "${apis[@]}"
+    fi
+}
+
+# Generates types_swagger_doc_generated file for the given group version.
+# $1: Name of the group version
+# $2: Path to the directory where types.go for that group version exists. This
+# is the directory where the file will be generated.
+function gen_types_swagger_doc() {
+    # The tool used to generate swagger code.
+    local swagger_bin
+    swagger_bin="$(kube::util::find-binary "genswaggertypedocs")"
+
+    local group_version="$1"
+    local gv_dir="$2"
+    local tmpfile
+    tmpfile="${TMPDIR:-/tmp}/types_swagger_doc_generated.$(date +%s).go"
+
+    if [[ "${DBG_CODEGEN}" == 1 ]]; then
+        kube::log::status "DBG: running ${swagger_bin} for ${group_version} at ${gv_dir}"
+    fi
+
+    {
+        cat "${BOILERPLATE_FILENAME}"
+        echo
+        echo "package ${group_version##*/}"
+        # Indenting here prevents the boilerplate checker from thinking this file
+        # is generated - gofmt will fix the indents anyway.
+        cat <<EOF
+
+          // This file contains a collection of methods that can be used from go-restful to
+          // generate Swagger API documentation for its models. Please read this PR for more
+          // information on the implementation: https://github.com/emicklei/go-restful/pull/215
+          //
+          // TODOs are ignored from the parser (e.g. TODO(andronat):... || TODO:...) if and only if
+          // they are on one line! For multiple line or blocks that you want to ignore use ---.
+          // Any context after a --- is ignored.
+          //
+          // Those methods can be generated by using hack/update-codegen.sh
+
+          // AUTO-GENERATED FUNCTIONS START HERE. DO NOT EDIT.
+EOF
+    } > "${tmpfile}"
+
+    "${swagger_bin}" \
+        -s \
+        "${gv_dir}/types.go" \
+        -f - \
+        >> "${tmpfile}"
+
+    echo "// AUTO-GENERATED FUNCTIONS END HERE" >> "${tmpfile}"
+
+    gofmt -w -s "${tmpfile}"
+    mv "${tmpfile}" "${gv_dir}/types_swagger_doc_generated.go"
+}
+
+# swagger generation
+#
+# Some of the later codegens depend on the results of this, so it needs to come
+# first in the case of regenerating everything.
+function codegen::swagger() {
+    # Build the tool
+    GO111MODULE=on GOPROXY=off go install \
+        ./cmd/genswaggertypedocs
+
+    local group_versions=()
+    IFS=" " read -r -a group_versions <<< "meta/v1 meta/v1beta1 ${KUBE_AVAILABLE_GROUP_VERSIONS}"
+
+    kube::log::status "Generating swagger for ${#group_versions[@]} targets"
+
+    git_find -z ':(glob)**/types_swagger_doc_generated.go' | xargs -0 rm -f
+
+    # Regenerate files.
+    for group_version in "${group_versions[@]}"; do
+      gen_types_swagger_doc "${group_version}" "$(kube::util::group-version-to-pkg-path "${group_version}")"
+    done
+}
+
 # prerelease-lifecycle generation
 #
 # Any package that wants prerelease-lifecycle functions generated must include a
@@ -116,13 +212,15 @@ fi
 #     // +k8s:prerelease-lifecycle-gen=true
 function codegen::prerelease() {
     # Build the tool.
-    hack/make-rules/build.sh k8s.io/code-generator/cmd/prerelease-lifecycle-gen
+    GO111MODULE=on GOPROXY=off go install \
+        k8s.io/code-generator/cmd/prerelease-lifecycle-gen
 
     # The result file, in each pkg, of prerelease-lifecycle generation.
     local output_base="${GENERATED_FILE_PREFIX}prerelease-lifecycle"
 
     # The tool used to generate prerelease-lifecycle code.
-    local gen_prerelease_bin="${BIN_DIR}/prerelease-lifecycle-gen"
+    local gen_prerelease_bin
+    gen_prerelease_bin="$(kube::util::find-binary "prerelease-lifecycle-gen")"
 
     # Find all the directories that request prerelease-lifecycle generation.
     if [[ "${DBG_CODEGEN}" == 1 ]]; then
@@ -130,9 +228,9 @@ function codegen::prerelease() {
     fi
     local tag_dirs=()
     kube::util::read-array tag_dirs < <( \
-        grep --color=never -l '+k8s:prerelease-lifecycle-gen=true' "${ALL_K8S_TAG_FILES[@]}" \
-            | xargs -n1 dirname \
-            | LC_ALL=C sort -u)
+        grep -l --null '+k8s:prerelease-lifecycle-gen=true' "${ALL_K8S_TAG_FILES[@]}" \
+            | xargs -0 -n1 dirname \
+            | sort -u)
     if [[ "${DBG_CODEGEN}" == 1 ]]; then
         kube::log::status "DBG: found ${#tag_dirs[@]} +k8s:prerelease-lifecycle-gen tagged dirs"
     fi
@@ -150,6 +248,8 @@ function codegen::prerelease() {
         done
     fi
 
+    git_find -z ':(glob)**'/"${output_base}.go" | xargs -0 rm -f
+
     ./hack/run-in-gopath.sh "${gen_prerelease_bin}" \
         --v "${KUBE_VERBOSE}" \
         --logtostderr \
@@ -175,13 +275,15 @@ function codegen::prerelease() {
 #               scheme
 function codegen::deepcopy() {
     # Build the tool.
-    hack/make-rules/build.sh k8s.io/code-generator/cmd/deepcopy-gen
+    GO111MODULE=on GOPROXY=off go install \
+        k8s.io/code-generator/cmd/deepcopy-gen
 
     # The result file, in each pkg, of deep-copy generation.
     local output_base="${GENERATED_FILE_PREFIX}deepcopy"
 
     # The tool used to generate deep copies.
-    local gen_deepcopy_bin="${BIN_DIR}/deepcopy-gen"
+    local gen_deepcopy_bin
+    gen_deepcopy_bin="$(kube::util::find-binary "deepcopy-gen")"
 
     # Find all the directories that request deep-copy generation.
     if [[ "${DBG_CODEGEN}" == 1 ]]; then
@@ -189,9 +291,9 @@ function codegen::deepcopy() {
     fi
     local tag_dirs=()
     kube::util::read-array tag_dirs < <( \
-        grep --color=never -l '+k8s:deepcopy-gen=' "${ALL_K8S_TAG_FILES[@]}" \
-            | xargs -n1 dirname \
-            | LC_ALL=C sort -u)
+        grep -l --null '+k8s:deepcopy-gen=' "${ALL_K8S_TAG_FILES[@]}" \
+            | xargs -0 -n1 dirname \
+            | sort -u)
     if [[ "${DBG_CODEGEN}" == 1 ]]; then
         kube::log::status "DBG: found ${#tag_dirs[@]} +k8s:deepcopy-gen tagged dirs"
     fi
@@ -209,6 +311,8 @@ function codegen::deepcopy() {
         done
     fi
 
+    git_find -z ':(glob)**'/"${output_base}.go" | xargs -0 rm -f
+
     ./hack/run-in-gopath.sh "${gen_deepcopy_bin}" \
         --v "${KUBE_VERBOSE}" \
         --logtostderr \
@@ -241,13 +345,15 @@ function codegen::deepcopy() {
 #                  for having a defaulter generated
 function codegen::defaults() {
     # Build the tool.
-    hack/make-rules/build.sh k8s.io/code-generator/cmd/defaulter-gen
+    GO111MODULE=on GOPROXY=off go install \
+        k8s.io/code-generator/cmd/defaulter-gen
 
     # The result file, in each pkg, of defaulter generation.
     local output_base="${GENERATED_FILE_PREFIX}defaults"
 
     # The tool used to generate defaulters.
-    local gen_defaulter_bin="${BIN_DIR}/defaulter-gen"
+    local gen_defaulter_bin
+    gen_defaulter_bin="$(kube::util::find-binary "defaulter-gen")"
 
     # All directories that request any form of defaulter generation.
     if [[ "${DBG_CODEGEN}" == 1 ]]; then
@@ -255,9 +361,9 @@ function codegen::defaults() {
     fi
     local tag_dirs=()
     kube::util::read-array tag_dirs < <( \
-        grep --color=never -l '+k8s:defaulter-gen=' "${ALL_K8S_TAG_FILES[@]}" \
-            | xargs -n1 dirname \
-            | LC_ALL=C sort -u)
+        grep -l --null '+k8s:defaulter-gen=' "${ALL_K8S_TAG_FILES[@]}" \
+            | xargs -0 -n1 dirname \
+            | sort -u)
     if [[ "${DBG_CODEGEN}" == 1 ]]; then
         kube::log::status "DBG: found ${#tag_dirs[@]} +k8s:defaulter-gen tagged dirs"
     fi
@@ -275,6 +381,8 @@ function codegen::defaults() {
         done
     fi
 
+    git_find -z ':(glob)**'/"${output_base}.go" | xargs -0 rm -f
+
     ./hack/run-in-gopath.sh "${gen_defaulter_bin}" \
         --v "${KUBE_VERBOSE}" \
         --logtostderr \
@@ -312,13 +420,15 @@ function codegen::defaults() {
 # IDL.
 function codegen::conversions() {
     # Build the tool.
-    hack/make-rules/build.sh k8s.io/code-generator/cmd/conversion-gen
+    GO111MODULE=on GOPROXY=off go install \
+        k8s.io/code-generator/cmd/conversion-gen
 
     # The result file, in each pkg, of conversion generation.
     local output_base="${GENERATED_FILE_PREFIX}conversion"
 
     # The tool used to generate conversions.
-    local gen_conversion_bin="${BIN_DIR}/conversion-gen"
+    local gen_conversion_bin
+    gen_conversion_bin="$(kube::util::find-binary "conversion-gen")"
 
     # All directories that request any form of conversion generation.
     if [[ "${DBG_CODEGEN}" == 1 ]]; then
@@ -326,9 +436,9 @@ function codegen::conversions() {
     fi
     local tag_dirs=()
     kube::util::read-array tag_dirs < <(\
-        grep --color=never -l '^// *+k8s:conversion-gen=' "${ALL_K8S_TAG_FILES[@]}" \
-            | xargs -n1 dirname \
-            | LC_ALL=C sort -u)
+        grep -l --null '^// *+k8s:conversion-gen=' "${ALL_K8S_TAG_FILES[@]}" \
+            | xargs -0 -n1 dirname \
+            | sort -u)
     if [[ "${DBG_CODEGEN}" == 1 ]]; then
         kube::log::status "DBG: found ${#tag_dirs[@]} +k8s:conversion-gen tagged dirs"
     fi
@@ -352,6 +462,8 @@ function codegen::conversions() {
         done
     fi
 
+    git_find -z ':(glob)**'/"${output_base}.go" | xargs -0 rm -f
+
     ./hack/run-in-gopath.sh "${gen_conversion_bin}" \
         --v "${KUBE_VERBOSE}" \
         --logtostderr \
@@ -433,13 +545,15 @@ function indirect_array() {
 #     // +k8s:openapi-gen=true
 function codegen::openapi() {
     # Build the tool.
-    hack/make-rules/build.sh k8s.io/kube-openapi/cmd/openapi-gen
+    GO111MODULE=on GOPROXY=off go install \
+        k8s.io/kube-openapi/cmd/openapi-gen
 
     # The result file, in each pkg, of open-api generation.
     local output_base="${GENERATED_FILE_PREFIX}openapi"
 
     # The tool used to generate open apis.
-    local gen_openapi_bin="${BIN_DIR}/openapi-gen"
+    local gen_openapi_bin
+    gen_openapi_bin="$(kube::util::find-binary "openapi-gen")"
 
     # Standard dirs which all targets need.
     local apimachinery_dirs=(
@@ -518,6 +632,8 @@ function codegen::openapi() {
             "${apimachinery_dirs[@]}"
         )
 
+    git_find -z ':(glob)**'/"${output_base}.go" | xargs -0 rm -f
+
     for prefix in "${targets[@]}"; do
         local report_file="${OUT_DIR}/${prefix}_violations.report"
         # When UPDATE_API_KNOWN_VIOLATIONS is set to be true, let the generator to write
@@ -536,10 +652,9 @@ function codegen::openapi() {
 
         local tag_dirs=()
         kube::util::read-array tag_dirs < <(
-            grep --color=never -l '+k8s:openapi-gen=' $(indirect_array "${prefix}_tag_files") \
-                | xargs -n1 dirname \
-                | LC_ALL=C sort -u
-            )
+            grep -l --null '+k8s:openapi-gen=' $(indirect_array "${prefix}_tag_files") \
+                | xargs -0 -n1 dirname \
+                | sort -u)
 
         if [[ "${DBG_CODEGEN}" == 1 ]]; then
             kube::log::status "DBG: found ${#tag_dirs[@]} +k8s:openapi-gen tagged dirs for ${prefix}"
@@ -558,7 +673,7 @@ function codegen::openapi() {
             done
         fi
 
-        ./hack/run-in-gopath.sh ${gen_openapi_bin} \
+        ./hack/run-in-gopath.sh "${gen_openapi_bin}" \
             --v "${KUBE_VERBOSE}" \
             --logtostderr \
             -h "${BOILERPLATE_FILENAME}" \
@@ -594,12 +709,12 @@ function codegen::applyconfigs() {
     local applyconfigurationgen
     applyconfigurationgen=$(kube::util::find-binary "applyconfiguration-gen")
 
-    # because client-gen doesn't do policy/v1alpha1, we have to skip it too
     local ext_apis=()
     kube::util::read-array ext_apis < <(
-      cd "${KUBE_ROOT}/staging/src"
-      find k8s.io/api -name types.go -print0 | xargs -0 -n1 dirname | sort | grep -v pkg.apis.policy.v1alpha1
-    )
+        cd "${KUBE_ROOT}/staging/src"
+        git_find -z ':(glob)k8s.io/api/**/types.go' \
+            | xargs -0 -n1 dirname \
+            | sort -u)
     ext_apis+=("k8s.io/apimachinery/pkg/apis/meta/v1")
 
     kube::log::status "Generating apply-config code for ${#ext_apis[@]} targets"
@@ -610,6 +725,12 @@ function codegen::applyconfigs() {
         done
     fi
 
+    git_grep -l --null \
+        -e '^// Code generated by applyconfiguration-gen. DO NOT EDIT.$' \
+        -- \
+        ':(glob)staging/src/k8s.io/client-go/**/*.go' \
+        | xargs -0 rm -f
+
     "${applyconfigurationgen}" \
         --openapi-schema <("${modelsschema}") \
         --go-header-file "${BOILERPLATE_FILENAME}" \
@@ -656,6 +777,12 @@ function codegen::clients() {
         done
     fi
 
+    git_grep -l --null \
+        -e '^// Code generated by client-gen. DO NOT EDIT.$' \
+        -- \
+        ':(glob)staging/src/k8s.io/client-go/**/*.go' \
+        | xargs -0 rm -f
+
     "${clientgen}" \
         --go-header-file "${BOILERPLATE_FILENAME}" \
         --output-base "${KUBE_ROOT}/vendor" \
@@ -672,7 +799,8 @@ function codegen::clients() {
 }
 
 function codegen::listers() {
-    GO111MODULE=on GOPROXY=off go install k8s.io/code-generator/cmd/lister-gen
+    GO111MODULE=on GOPROXY=off go install \
+        k8s.io/code-generator/cmd/lister-gen
 
     local listergen
     listergen=$(kube::util::find-binary "lister-gen")
@@ -680,8 +808,9 @@ function codegen::listers() {
     local ext_apis=()
     kube::util::read-array ext_apis < <(
         cd "${KUBE_ROOT}/staging/src"
-        find k8s.io/api -name types.go -print0 | xargs -0 -n1 dirname | sort
-    )
+        git_find -z ':(glob)k8s.io/api/**/types.go' \
+            | xargs -0 -n1 dirname \
+            | sort -u)
 
     kube::log::status "Generating lister code for ${#ext_apis[@]} targets"
     if [[ "${DBG_CODEGEN}" == 1 ]]; then
@@ -691,6 +820,12 @@ function codegen::listers() {
         done
     fi
 
+    git_grep -l --null \
+        -e '^// Code generated by lister-gen. DO NOT EDIT.$' \
+        -- \
+        ':(glob)staging/src/k8s.io/client-go/**/*.go' \
+        | xargs -0 rm -f
+
     "${listergen}" \
         --go-header-file "${BOILERPLATE_FILENAME}" \
         --output-base "${KUBE_ROOT}/vendor" \
@@ -710,12 +845,12 @@ function codegen::informers() {
     local informergen
     informergen=$(kube::util::find-binary "informer-gen")
 
-    # because client-gen doesn't do policy/v1alpha1, we have to skip it too
     local ext_apis=()
     kube::util::read-array ext_apis < <(
         cd "${KUBE_ROOT}/staging/src"
-        find k8s.io/api -name types.go -print0 | xargs -0 -n1 dirname | sort | grep -v pkg.apis.policy.v1alpha1
-    )
+        git_find -z ':(glob)k8s.io/api/**/types.go' \
+            | xargs -0 -n1 dirname \
+            | sort -u)
 
     kube::log::status "Generating informer code for ${#ext_apis[@]} targets"
     if [[ "${DBG_CODEGEN}" == 1 ]]; then
@@ -725,6 +860,12 @@ function codegen::informers() {
         done
     fi
 
+    git_grep -l --null \
+        -e '^// Code generated by informer-gen. DO NOT EDIT.$' \
+        -- \
+        ':(glob)staging/src/k8s.io/client-go/**/*.go' \
+        | xargs -0 rm -f
+
     "${informergen}" \
         --go-header-file "${BOILERPLATE_FILENAME}" \
         --output-base "${KUBE_ROOT}/vendor" \
@@ -758,6 +899,48 @@ function codegen::subprojects() {
     done
 }
 
+function codegen::protobindings() {
+    # Each element of this array is a directory containing subdirectories which
+    # eventually contain a file named "api.proto".
+    local apis=(
+        "staging/src/k8s.io/cri-api/pkg/apis/runtime"
+
+        "staging/src/k8s.io/kubelet/pkg/apis/podresources"
+
+        "staging/src/k8s.io/kubelet/pkg/apis/deviceplugin"
+
+        "staging/src/k8s.io/kms/apis"
+        "staging/src/k8s.io/apiserver/pkg/storage/value/encrypt/envelope/kmsv2"
+
+        "staging/src/k8s.io/kubelet/pkg/apis/dra"
+
+        "staging/src/k8s.io/kubelet/pkg/apis/pluginregistration"
+        "pkg/kubelet/pluginmanager/pluginwatcher/example_plugin_apis"
+    )
+
+    kube::log::status "Generating protobuf bindings for ${#apis[@]} targets"
+    if [[ "${DBG_CODEGEN}" == 1 ]]; then
+        kube::log::status "DBG: generating protobuf bindings for:"
+        for dir in "${apis[@]}"; do
+            kube::log::status "DBG:     $dir"
+        done
+    fi
+
+    for api in "${apis[@]}"; do
+        git ls-files -z -cmo --exclude-standard ":(glob)${api}"/'**/api.pb.go' \
+            | xargs -0 rm -f
+    done
+
+    if kube::protoc::check_protoc >/dev/null; then
+      hack/update-generated-proto-bindings-dockerized.sh "${apis[@]}"
+    else
+      kube::log::status "protoc ${PROTOC_VERSION} not found (can install with hack/install-protoc.sh); generating containerized..."
+      # NOTE: All output from this script needs to be copied back to the calling
+      # source tree.  This is managed in kube::build::copy_output in build/common.sh.
+      # If the output set is changed update that function.
+      build/run.sh hack/update-generated-proto-bindings-dockerized.sh "${apis[@]}"
+    fi
+}
 
 #
 # main
@@ -820,7 +1003,8 @@ for arg; do
         print_codegens
         exit 1
     fi
-    codegens_to_run+=("${matches[@]}")
+    # The array-syntax abomination is to accommodate older bash.
+    codegens_to_run+=("${matches[@]:+"${matches[@]}"}")
 done
 
 # If no codegens were specified, run them all.
@@ -829,5 +1013,6 @@ if [[ "${#codegens_to_run[@]}" == 0 ]]; then
 fi
 
 for g in "${codegens_to_run[@]}"; do
-    "codegen::${g}" "${flags_to_pass[@]}"
+    # The array-syntax abomination is to accommodate older bash.
+    "codegen::${g}" "${flags_to_pass[@]:+"${flags_to_pass[@]}"}"
 done
