@@ -85,11 +85,11 @@ func TestSetDefaultJob(t *testing.T) {
 			},
 			expected: &batchv1.Job{
 				Spec: batchv1.JobSpec{
-					Completions:    pointer.Int32Ptr(1),
-					Parallelism:    pointer.Int32Ptr(1),
-					BackoffLimit:   pointer.Int32Ptr(6),
+					Completions:    pointer.Int32(1),
+					Parallelism:    pointer.Int32(1),
+					BackoffLimit:   pointer.Int32(6),
 					CompletionMode: completionModePtr(batchv1.NonIndexedCompletion),
-					Suspend:        pointer.BoolPtr(false),
+					Suspend:        pointer.Bool(false),
 					PodFailurePolicy: &batchv1.PodFailurePolicy{
 						Rules: []batchv1.PodFailurePolicyRule{
 							{
@@ -141,11 +141,11 @@ func TestSetDefaultJob(t *testing.T) {
 			},
 			expected: &batchv1.Job{
 				Spec: batchv1.JobSpec{
-					Completions:    pointer.Int32Ptr(1),
-					Parallelism:    pointer.Int32Ptr(1),
-					BackoffLimit:   pointer.Int32Ptr(6),
+					Completions:    pointer.Int32(1),
+					Parallelism:    pointer.Int32(1),
+					BackoffLimit:   pointer.Int32(6),
 					CompletionMode: completionModePtr(batchv1.NonIndexedCompletion),
-					Suspend:        pointer.BoolPtr(false),
+					Suspend:        pointer.Bool(false),
 				},
 			},
 			expectLabels: true,
@@ -160,11 +160,11 @@ func TestSetDefaultJob(t *testing.T) {
 			},
 			expected: &batchv1.Job{
 				Spec: batchv1.JobSpec{
-					Completions:    pointer.Int32Ptr(1),
-					Parallelism:    pointer.Int32Ptr(1),
-					BackoffLimit:   pointer.Int32Ptr(6),
+					Completions:    pointer.Int32(1),
+					Parallelism:    pointer.Int32(1),
+					BackoffLimit:   pointer.Int32(6),
 					CompletionMode: completionModePtr(batchv1.NonIndexedCompletion),
-					Suspend:        pointer.BoolPtr(false),
+					Suspend:        pointer.Bool(false),
 				},
 			},
 			expectLabels: true,
@@ -172,7 +172,7 @@ func TestSetDefaultJob(t *testing.T) {
 		"suspend set, everything else is defaulted": {
 			original: &batchv1.Job{
 				Spec: batchv1.JobSpec{
-					Suspend: pointer.BoolPtr(true),
+					Suspend: pointer.Bool(true),
 					Template: v1.PodTemplateSpec{
 						ObjectMeta: metav1.ObjectMeta{Labels: defaultLabels},
 					},
@@ -180,11 +180,11 @@ func TestSetDefaultJob(t *testing.T) {
 			},
 			expected: &batchv1.Job{
 				Spec: batchv1.JobSpec{
-					Completions:    pointer.Int32Ptr(1),
-					Parallelism:    pointer.Int32Ptr(1),
-					BackoffLimit:   pointer.Int32Ptr(6),
+					Completions:    pointer.Int32(1),
+					Parallelism:    pointer.Int32(1),
+					BackoffLimit:   pointer.Int32(6),
 					CompletionMode: completionModePtr(batchv1.NonIndexedCompletion),
-					Suspend:        pointer.BoolPtr(true),
+					Suspend:        pointer.Bool(true),
 				},
 			},
 			expectLabels: true,
@@ -202,18 +202,18 @@ func TestSetDefaultJob(t *testing.T) {
 			},
 			expected: &batchv1.Job{
 				Spec: batchv1.JobSpec{
-					Completions:    pointer.Int32Ptr(1),
-					Parallelism:    pointer.Int32Ptr(1),
-					BackoffLimit:   pointer.Int32Ptr(6),
+					Completions:    pointer.Int32(1),
+					Parallelism:    pointer.Int32(1),
+					BackoffLimit:   pointer.Int32(6),
 					CompletionMode: completionModePtr(batchv1.NonIndexedCompletion),
-					Suspend:        pointer.BoolPtr(false),
+					Suspend:        pointer.Bool(false),
 				},
 			},
 		},
 		"WQ: Parallelism explicitly 0 and completions unset -> BackoffLimit is defaulted": {
 			original: &batchv1.Job{
 				Spec: batchv1.JobSpec{
-					Parallelism: pointer.Int32Ptr(0),
+					Parallelism: pointer.Int32(0),
 					Template: v1.PodTemplateSpec{
 						ObjectMeta: metav1.ObjectMeta{Labels: defaultLabels},
 					},
@@ -221,10 +221,10 @@ func TestSetDefaultJob(t *testing.T) {
 			},
 			expected: &batchv1.Job{
 				Spec: batchv1.JobSpec{
-					Parallelism:    pointer.Int32Ptr(0),
-					BackoffLimit:   pointer.Int32Ptr(6),
+					Parallelism:    pointer.Int32(0),
+					BackoffLimit:   pointer.Int32(6),
 					CompletionMode: completionModePtr(batchv1.NonIndexedCompletion),
-					Suspend:        pointer.BoolPtr(false),
+					Suspend:        pointer.Bool(false),
 				},
 			},
 			expectLabels: true,
@@ -232,7 +232,7 @@ func TestSetDefaultJob(t *testing.T) {
 		"WQ: Parallelism explicitly 2 and completions unset -> BackoffLimit is defaulted": {
 			original: &batchv1.Job{
 				Spec: batchv1.JobSpec{
-					Parallelism: pointer.Int32Ptr(2),
+					Parallelism: pointer.Int32(2),
 					Template: v1.PodTemplateSpec{
 						ObjectMeta: metav1.ObjectMeta{Labels: defaultLabels},
 					},
@@ -240,10 +240,10 @@ func TestSetDefaultJob(t *testing.T) {
 			},
 			expected: &batchv1.Job{
 				Spec: batchv1.JobSpec{
-					Parallelism:    pointer.Int32Ptr(2),
-					BackoffLimit:   pointer.Int32Ptr(6),
+					Parallelism:    pointer.Int32(2),
+					BackoffLimit:   pointer.Int32(6),
 					CompletionMode: completionModePtr(batchv1.NonIndexedCompletion),
-					Suspend:        pointer.BoolPtr(false),
+					Suspend:        pointer.Bool(false),
 				},
 			},
 			expectLabels: true,
@@ -251,7 +251,7 @@ func TestSetDefaultJob(t *testing.T) {
 		"Completions explicitly 2 and others unset -> parallelism and BackoffLimit are defaulted": {
 			original: &batchv1.Job{
 				Spec: batchv1.JobSpec{
-					Completions: pointer.Int32Ptr(2),
+					Completions: pointer.Int32(2),
 					Template: v1.PodTemplateSpec{
 						ObjectMeta: metav1.ObjectMeta{Labels: defaultLabels},
 					},
@@ -259,11 +259,11 @@ func TestSetDefaultJob(t *testing.T) {
 			},
 			expected: &batchv1.Job{
 				Spec: batchv1.JobSpec{
-					Completions:    pointer.Int32Ptr(2),
-					Parallelism:    pointer.Int32Ptr(1),
-					BackoffLimit:   pointer.Int32Ptr(6),
+					Completions:    pointer.Int32(2),
+					Parallelism:    pointer.Int32(1),
+					BackoffLimit:   pointer.Int32(6),
 					CompletionMode: completionModePtr(batchv1.NonIndexedCompletion),
-					Suspend:        pointer.BoolPtr(false),
+					Suspend:        pointer.Bool(false),
 				},
 			},
 			expectLabels: true,
@@ -271,7 +271,7 @@ func TestSetDefaultJob(t *testing.T) {
 		"BackoffLimit explicitly 5 and others unset -> parallelism and completions are defaulted": {
 			original: &batchv1.Job{
 				Spec: batchv1.JobSpec{
-					BackoffLimit: pointer.Int32Ptr(5),
+					BackoffLimit: pointer.Int32(5),
 					Template: v1.PodTemplateSpec{
 						ObjectMeta: metav1.ObjectMeta{Labels: defaultLabels},
 					},
@@ -279,11 +279,11 @@ func TestSetDefaultJob(t *testing.T) {
 			},
 			expected: &batchv1.Job{
 				Spec: batchv1.JobSpec{
-					Completions:    pointer.Int32Ptr(1),
-					Parallelism:    pointer.Int32Ptr(1),
-					BackoffLimit:   pointer.Int32Ptr(5),
+					Completions:    pointer.Int32(1),
+					Parallelism:    pointer.Int32(1),
+					BackoffLimit:   pointer.Int32(5),
 					CompletionMode: completionModePtr(batchv1.NonIndexedCompletion),
-					Suspend:        pointer.BoolPtr(false),
+					Suspend:        pointer.Bool(false),
 				},
 			},
 			expectLabels: true,
@@ -291,11 +291,11 @@ func TestSetDefaultJob(t *testing.T) {
 		"All set -> no change": {
 			original: &batchv1.Job{
 				Spec: batchv1.JobSpec{
-					Completions:    pointer.Int32Ptr(8),
-					Parallelism:    pointer.Int32Ptr(9),
-					BackoffLimit:   pointer.Int32Ptr(10),
+					Completions:    pointer.Int32(8),
+					Parallelism:    pointer.Int32(9),
+					BackoffLimit:   pointer.Int32(10),
 					CompletionMode: completionModePtr(batchv1.NonIndexedCompletion),
-					Suspend:        pointer.BoolPtr(false),
+					Suspend:        pointer.Bool(false),
 					Template: v1.PodTemplateSpec{
 						ObjectMeta: metav1.ObjectMeta{Labels: defaultLabels},
 					},
@@ -303,11 +303,11 @@ func TestSetDefaultJob(t *testing.T) {
 			},
 			expected: &batchv1.Job{
 				Spec: batchv1.JobSpec{
-					Completions:    pointer.Int32Ptr(8),
-					Parallelism:    pointer.Int32Ptr(9),
-					BackoffLimit:   pointer.Int32Ptr(10),
+					Completions:    pointer.Int32(8),
+					Parallelism:    pointer.Int32(9),
+					BackoffLimit:   pointer.Int32(10),
 					CompletionMode: completionModePtr(batchv1.NonIndexedCompletion),
-					Suspend:        pointer.BoolPtr(false),
+					Suspend:        pointer.Bool(false),
 					Template: v1.PodTemplateSpec{
 						ObjectMeta: metav1.ObjectMeta{Labels: defaultLabels},
 					},
@@ -318,11 +318,11 @@ func TestSetDefaultJob(t *testing.T) {
 		"All set, flipped -> no change": {
 			original: &batchv1.Job{
 				Spec: batchv1.JobSpec{
-					Completions:    pointer.Int32Ptr(11),
-					Parallelism:    pointer.Int32Ptr(10),
-					BackoffLimit:   pointer.Int32Ptr(9),
+					Completions:    pointer.Int32(11),
+					Parallelism:    pointer.Int32(10),
+					BackoffLimit:   pointer.Int32(9),
 					CompletionMode: completionModePtr(batchv1.IndexedCompletion),
-					Suspend:        pointer.BoolPtr(true),
+					Suspend:        pointer.Bool(true),
 					Template: v1.PodTemplateSpec{
 						ObjectMeta: metav1.ObjectMeta{Labels: defaultLabels},
 					},
@@ -330,11 +330,11 @@ func TestSetDefaultJob(t *testing.T) {
 			},
 			expected: &batchv1.Job{
 				Spec: batchv1.JobSpec{
-					Completions:    pointer.Int32Ptr(11),
-					Parallelism:    pointer.Int32Ptr(10),
-					BackoffLimit:   pointer.Int32Ptr(9),
+					Completions:    pointer.Int32(11),
+					Parallelism:    pointer.Int32(10),
+					BackoffLimit:   pointer.Int32(9),
 					CompletionMode: completionModePtr(batchv1.IndexedCompletion),
-					Suspend:        pointer.BoolPtr(true),
+					Suspend:        pointer.Bool(true),
 				},
 			},
 			expectLabels: true,
@@ -416,9 +416,9 @@ func TestSetDefaultCronJob(t *testing.T) {
 			expected: &batchv1.CronJob{
 				Spec: batchv1.CronJobSpec{
 					ConcurrencyPolicy:          batchv1.AllowConcurrent,
-					Suspend:                    pointer.BoolPtr(false),
-					SuccessfulJobsHistoryLimit: pointer.Int32Ptr(3),
-					FailedJobsHistoryLimit:     pointer.Int32Ptr(1),
+					Suspend:                    pointer.Bool(false),
+					SuccessfulJobsHistoryLimit: pointer.Int32(3),
+					FailedJobsHistoryLimit:     pointer.Int32(1),
 				},
 			},
 		},
@@ -426,17 +426,17 @@ func TestSetDefaultCronJob(t *testing.T) {
 			original: &batchv1.CronJob{
 				Spec: batchv1.CronJobSpec{
 					ConcurrencyPolicy:          batchv1.ForbidConcurrent,
-					Suspend:                    pointer.BoolPtr(true),
-					SuccessfulJobsHistoryLimit: pointer.Int32Ptr(5),
-					FailedJobsHistoryLimit:     pointer.Int32Ptr(5),
+					Suspend:                    pointer.Bool(true),
+					SuccessfulJobsHistoryLimit: pointer.Int32(5),
+					FailedJobsHistoryLimit:     pointer.Int32(5),
 				},
 			},
 			expected: &batchv1.CronJob{
 				Spec: batchv1.CronJobSpec{
 					ConcurrencyPolicy:          batchv1.ForbidConcurrent,
-					Suspend:                    pointer.BoolPtr(true),
-					SuccessfulJobsHistoryLimit: pointer.Int32Ptr(5),
-					FailedJobsHistoryLimit:     pointer.Int32Ptr(5),
+					Suspend:                    pointer.Bool(true),
+					SuccessfulJobsHistoryLimit: pointer.Int32(5),
+					FailedJobsHistoryLimit:     pointer.Int32(5),
 				},
 			},
 		},
