@@ -25,10 +25,11 @@ import (
 
 	cron "github.com/robfig/cron/v3"
 	batchv1 "k8s.io/api/batch/v1"
-	v1 "k8s.io/api/core/v1"
+	"k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/client-go/tools/record"
+	"k8s.io/klog/v2/ktesting"
 )
 
 func TestGetJobFromTemplate2(t *testing.T) {
@@ -89,10 +90,11 @@ func TestGetJobFromTemplate2(t *testing.T) {
 }
 
 func TestNextScheduleTime(t *testing.T) {
+	logger, _ := ktesting.NewTestContext(t)
 	// schedule is hourly on the hour
 	schedule := "0 * * * ?"
 
-	ParseSchedule := func(schedule string) cron.Schedule {
+	PraseSchedule := func(schedule string) cron.Schedule {
 		sched, err := cron.ParseStandard(schedule)
 		if err != nil {
 			t.Errorf("Error parsing schedule: %#v", err)
@@ -124,7 +126,7 @@ func TestNextScheduleTime(t *testing.T) {
 		cj.ObjectMeta.CreationTimestamp = metav1.Time{Time: T1.Add(-10 * time.Minute)}
 		// Current time is more than creation time, but less than T1.
 		now := T1.Add(-7 * time.Minute)
-		schedule, _ := nextScheduleTime(&cj, now, ParseSchedule(cj.Spec.Schedule), recorder)
+		schedule, _ := nextScheduleTime(logger, &cj, now, PraseSchedule(cj.Spec.Schedule), recorder)
 		if schedule != nil {
 			t.Errorf("expected no start time, got:  %v", schedule)
 		}
@@ -135,7 +137,7 @@ func TestNextScheduleTime(t *testing.T) {
 		cj.ObjectMeta.CreationTimestamp = metav1.Time{Time: T1.Add(-10 * time.Minute)}
 		// Current time is after T1
 		now := T1.Add(2 * time.Second)
-		schedule, _ := nextScheduleTime(&cj, now, ParseSchedule(cj.Spec.Schedule), recorder)
+		schedule, _ := nextScheduleTime(logger, &cj, now, PraseSchedule(cj.Spec.Schedule), recorder)
 		if schedule == nil {
 			t.Errorf("expected 1 start time, got nil")
 		} else if !schedule.Equal(T1) {
@@ -150,7 +152,7 @@ func TestNextScheduleTime(t *testing.T) {
 		cj.Status.LastScheduleTime = &metav1.Time{Time: T1}
 		// Current time is after T1
 		now := T1.Add(2 * time.Minute)
-		schedule, _ := nextScheduleTime(&cj, now, ParseSchedule(cj.Spec.Schedule), recorder)
+		schedule, _ := nextScheduleTime(logger, &cj, now, PraseSchedule(cj.Spec.Schedule), recorder)
 		if schedule != nil {
 			t.Errorf("expected 0 start times, got: %v", schedule)
 		}
@@ -163,7 +165,7 @@ func TestNextScheduleTime(t *testing.T) {
 		cj.Status.LastScheduleTime = &metav1.Time{Time: T1}
 		// Current time is after T1 and after T2
 		now := T2.Add(5 * time.Minute)
-		schedule, _ := nextScheduleTime(&cj, now, ParseSchedule(cj.Spec.Schedule), recorder)
+		schedule, _ := nextScheduleTime(logger, &cj, now, PraseSchedule(cj.Spec.Schedule), recorder)
 		if schedule == nil {
 			t.Errorf("expected 1 start times, got nil")
 		} else if !schedule.Equal(T2) {
@@ -176,7 +178,7 @@ func TestNextScheduleTime(t *testing.T) {
 		cj.Status.LastScheduleTime = &metav1.Time{Time: T1.Add(-1 * time.Hour)}
 		// Current time is after T1 and after T2
 		now := T2.Add(5 * time.Minute)
-		schedule, _ := nextScheduleTime(&cj, now, ParseSchedule(cj.Spec.Schedule), recorder)
+		schedule, _ := nextScheduleTime(logger, &cj, now, PraseSchedule(cj.Spec.Schedule), recorder)
 		if schedule == nil {
 			t.Errorf("expected 1 start times, got nil")
 		} else if !schedule.Equal(T2) {
@@ -188,7 +190,7 @@ func TestNextScheduleTime(t *testing.T) {
 		cj.ObjectMeta.CreationTimestamp = metav1.Time{Time: T1.Add(-2 * time.Hour)}
 		cj.Status.LastScheduleTime = &metav1.Time{Time: T1.Add(-1 * time.Hour)}
 		now := T2.Add(10 * 24 * time.Hour)
-		schedule, _ := nextScheduleTime(&cj, now, ParseSchedule(cj.Spec.Schedule), recorder)
+		schedule, _ := nextScheduleTime(logger, &cj, now, PraseSchedule(cj.Spec.Schedule), recorder)
 		if schedule == nil {
 			t.Errorf("expected more than 0 missed times")
 		}
@@ -201,7 +203,7 @@ func TestNextScheduleTime(t *testing.T) {
 		// Deadline is short
 		deadline := int64(2 * 60 * 60)
 		cj.Spec.StartingDeadlineSeconds = &deadline
-		schedule, _ := nextScheduleTime(&cj, now, ParseSchedule(cj.Spec.Schedule), recorder)
+		schedule, _ := nextScheduleTime(logger, &cj, now, PraseSchedule(cj.Spec.Schedule), recorder)
 		if schedule == nil {
 			t.Errorf("expected more than 0 missed times")
 		}
@@ -212,7 +214,7 @@ func TestNextScheduleTime(t *testing.T) {
 		cj.Status.LastScheduleTime = nil
 		now := *deltaTimeAfterTopOfTheHour(1 * time.Hour)
 		// rouge schedule
-		schedule, err := nextScheduleTime(&cj, now, ParseSchedule("59 23 31 2 *"), recorder)
+		schedule, err := nextScheduleTime(logger, &cj, now, PraseSchedule("59 23 31 2 *"), recorder)
 		if schedule != nil {
 			t.Errorf("expected no start time, got:  %v", schedule)
 		}
@@ -288,19 +290,17 @@ func TestByJobStartTime(t *testing.T) {
 func TestMostRecentScheduleTime(t *testing.T) {
 	metav1TopOfTheHour := metav1.NewTime(*topOfTheHour())
 	metav1HalfPastTheHour := metav1.NewTime(*deltaTimeAfterTopOfTheHour(30 * time.Minute))
-	metav1MinuteAfterTopOfTheHour := metav1.NewTime(*deltaTimeAfterTopOfTheHour(1 * time.Minute))
 	oneMinute := int64(60)
-	tenSeconds := int64(10)
 
 	tests := []struct {
-		name                  string
-		cj                    *batchv1.CronJob
-		includeSDS            bool
-		now                   time.Time
-		expectedEarliestTime  time.Time
-		expectedRecentTime    *time.Time
-		expectedTooManyMissed missedSchedulesType
-		wantErr               bool
+		name                   string
+		cj                     *batchv1.CronJob
+		includeSDS             bool
+		now                    time.Time
+		expectedEarliestTime   time.Time
+		expectedRecentTime     *time.Time
+		expectedNumberOfMisses int64
+		wantErr                bool
 	}{
 		{
 			name: "now before next schedule",
@@ -326,9 +326,10 @@ func TestMostRecentScheduleTime(t *testing.T) {
 					Schedule: "0 * * * *",
 				},
 			},
-			now:                  topOfTheHour().Add(61 * time.Minute),
-			expectedRecentTime:   deltaTimeAfterTopOfTheHour(60 * time.Minute),
-			expectedEarliestTime: *topOfTheHour(),
+			now:                    topOfTheHour().Add(61 * time.Minute),
+			expectedRecentTime:     deltaTimeAfterTopOfTheHour(60 * time.Minute),
+			expectedEarliestTime:   *topOfTheHour(),
+			expectedNumberOfMisses: 1,
 		},
 		{
 			name: "missed 5 schedules",
@@ -340,98 +341,10 @@ func TestMostRecentScheduleTime(t *testing.T) {
 					Schedule: "0 * * * *",
 				},
 			},
-			now:                   *deltaTimeAfterTopOfTheHour(301 * time.Minute),
-			expectedRecentTime:    deltaTimeAfterTopOfTheHour(300 * time.Minute),
-			expectedEarliestTime:  *deltaTimeAfterTopOfTheHour(10 * time.Second),
-			expectedTooManyMissed: fewMissed,
-		},
-		{
-			name: "complex schedule",
-			cj: &batchv1.CronJob{
-				ObjectMeta: metav1.ObjectMeta{
-					CreationTimestamp: metav1TopOfTheHour,
-				},
-				Spec: batchv1.CronJobSpec{
-					Schedule: "30 6-16/4 * * 1-5",
-				},
-				Status: batchv1.CronJobStatus{
-					LastScheduleTime: &metav1HalfPastTheHour,
-				},
-			},
-			now:                   *deltaTimeAfterTopOfTheHour(24*time.Hour + 31*time.Minute),
-			expectedRecentTime:    deltaTimeAfterTopOfTheHour(24*time.Hour + 30*time.Minute),
-			expectedEarliestTime:  *deltaTimeAfterTopOfTheHour(30 * time.Minute),
-			expectedTooManyMissed: fewMissed,
-		},
-		{
-			name: "another complex schedule",
-			cj: &batchv1.CronJob{
-				ObjectMeta: metav1.ObjectMeta{
-					CreationTimestamp: metav1TopOfTheHour,
-				},
-				Spec: batchv1.CronJobSpec{
-					Schedule: "30 10,11,12 * * 1-5",
-				},
-				Status: batchv1.CronJobStatus{
-					LastScheduleTime: &metav1HalfPastTheHour,
-				},
-			},
-			now:                   *deltaTimeAfterTopOfTheHour(30*time.Hour + 30*time.Minute),
-			expectedRecentTime:    nil,
-			expectedEarliestTime:  *deltaTimeAfterTopOfTheHour(30 * time.Minute),
-			expectedTooManyMissed: fewMissed,
-		},
-		{
-			name: "complex schedule with longer diff between executions",
-			cj: &batchv1.CronJob{
-				ObjectMeta: metav1.ObjectMeta{
-					CreationTimestamp: metav1TopOfTheHour,
-				},
-				Spec: batchv1.CronJobSpec{
-					Schedule: "30 6-16/4 * * 1-5",
-				},
-				Status: batchv1.CronJobStatus{
-					LastScheduleTime: &metav1HalfPastTheHour,
-				},
-			},
-			now:                   *deltaTimeAfterTopOfTheHour(96*time.Hour + 31*time.Minute),
-			expectedRecentTime:    deltaTimeAfterTopOfTheHour(96*time.Hour + 30*time.Minute),
-			expectedEarliestTime:  *deltaTimeAfterTopOfTheHour(30 * time.Minute),
-			expectedTooManyMissed: fewMissed,
-		},
-		{
-			name: "complex schedule with shorter diff between executions",
-			cj: &batchv1.CronJob{
-				ObjectMeta: metav1.ObjectMeta{
-					CreationTimestamp: metav1TopOfTheHour,
-				},
-				Spec: batchv1.CronJobSpec{
-					Schedule: "30 6-16/4 * * 1-5",
-				},
-			},
-			now:                   *deltaTimeAfterTopOfTheHour(24*time.Hour + 31*time.Minute),
-			expectedRecentTime:    deltaTimeAfterTopOfTheHour(24*time.Hour + 30*time.Minute),
-			expectedEarliestTime:  *topOfTheHour(),
-			expectedTooManyMissed: fewMissed,
-		},
-		{
-			name: "@every schedule",
-			cj: &batchv1.CronJob{
-				ObjectMeta: metav1.ObjectMeta{
-					CreationTimestamp: metav1.NewTime(*deltaTimeAfterTopOfTheHour(-59 * time.Minute)),
-				},
-				Spec: batchv1.CronJobSpec{
-					Schedule:                "@every 1h",
-					StartingDeadlineSeconds: &tenSeconds,
-				},
-				Status: batchv1.CronJobStatus{
-					LastScheduleTime: &metav1MinuteAfterTopOfTheHour,
-				},
-			},
-			now:                   *deltaTimeAfterTopOfTheHour(7 * 24 * time.Hour),
-			expectedRecentTime:    deltaTimeAfterTopOfTheHour((6 * 24 * time.Hour) + 23*time.Hour + 1*time.Minute),
-			expectedEarliestTime:  *deltaTimeAfterTopOfTheHour(1 * time.Minute),
-			expectedTooManyMissed: manyMissed,
+			now:                    *deltaTimeAfterTopOfTheHour(301 * time.Minute),
+			expectedRecentTime:     deltaTimeAfterTopOfTheHour(300 * time.Minute),
+			expectedEarliestTime:   *deltaTimeAfterTopOfTheHour(10 * time.Second),
+			expectedNumberOfMisses: 5,
 		},
 		{
 			name: "rogue cronjob",
@@ -443,9 +356,10 @@ func TestMostRecentScheduleTime(t *testing.T) {
 					Schedule: "59 23 31 2 *",
 				},
 			},
-			now:                *deltaTimeAfterTopOfTheHour(1 * time.Hour),
-			expectedRecentTime: nil,
-			wantErr:            true,
+			now:                    *deltaTimeAfterTopOfTheHour(1 * time.Hour),
+			expectedRecentTime:     nil,
+			expectedNumberOfMisses: 0,
+			wantErr:                true,
 		},
 		{
 			name: "earliestTime being CreationTimestamp and LastScheduleTime",
@@ -544,98 +458,8 @@ func TestMostRecentScheduleTime(t *testing.T) {
 			if !reflect.DeepEqual(gotRecentTime, tt.expectedRecentTime) {
 				t.Errorf("expectedRecentTime - got %v, want %v", gotRecentTime, tt.expectedRecentTime)
 			}
-			if gotNumberOfMisses != tt.expectedTooManyMissed {
-				t.Errorf("expectedNumberOfMisses - got %v, want %v", gotNumberOfMisses, tt.expectedTooManyMissed)
-			}
-		})
-	}
-}
-
-func TestNextScheduleTimeDuration(t *testing.T) {
-	metav1TopOfTheHour := metav1.NewTime(*topOfTheHour())
-	metav1HalfPastTheHour := metav1.NewTime(*deltaTimeAfterTopOfTheHour(30 * time.Minute))
-	metav1TwoHoursLater := metav1.NewTime(*deltaTimeAfterTopOfTheHour(2 * time.Hour))
-
-	tests := []struct {
-		name             string
-		cj               *batchv1.CronJob
-		now              time.Time
-		expectedDuration time.Duration
-	}{
-		{
-			name: "complex schedule skipping weekend",
-			cj: &batchv1.CronJob{
-				ObjectMeta: metav1.ObjectMeta{
-					CreationTimestamp: metav1TopOfTheHour,
-				},
-				Spec: batchv1.CronJobSpec{
-					Schedule: "30 6-16/4 * * 1-5",
-				},
-				Status: batchv1.CronJobStatus{
-					LastScheduleTime: &metav1HalfPastTheHour,
-				},
-			},
-			now:              *deltaTimeAfterTopOfTheHour(24*time.Hour + 31*time.Minute),
-			expectedDuration: 3*time.Hour + 59*time.Minute + nextScheduleDelta,
-		},
-		{
-			name: "another complex schedule skipping weekend",
-			cj: &batchv1.CronJob{
-				ObjectMeta: metav1.ObjectMeta{
-					CreationTimestamp: metav1TopOfTheHour,
-				},
-				Spec: batchv1.CronJobSpec{
-					Schedule: "30 10,11,12 * * 1-5",
-				},
-				Status: batchv1.CronJobStatus{
-					LastScheduleTime: &metav1HalfPastTheHour,
-				},
-			},
-			now:              *deltaTimeAfterTopOfTheHour(30*time.Hour + 30*time.Minute),
-			expectedDuration: 66*time.Hour + nextScheduleDelta,
-		},
-		{
-			name: "once a week cronjob, missed two runs",
-			cj: &batchv1.CronJob{
-				ObjectMeta: metav1.ObjectMeta{
-					CreationTimestamp: metav1TopOfTheHour,
-				},
-				Spec: batchv1.CronJobSpec{
-					Schedule: "0 12 * * 4",
-				},
-				Status: batchv1.CronJobStatus{
-					LastScheduleTime: &metav1TwoHoursLater,
-				},
-			},
-			now:              *deltaTimeAfterTopOfTheHour(19*24*time.Hour + 1*time.Hour + 30*time.Minute),
-			expectedDuration: 48*time.Hour + 30*time.Minute + nextScheduleDelta,
-		},
-		{
-			name: "no previous run of a cronjob",
-			cj: &batchv1.CronJob{
-				ObjectMeta: metav1.ObjectMeta{
-					CreationTimestamp: metav1TopOfTheHour,
-				},
-				Spec: batchv1.CronJobSpec{
-					Schedule: "0 12 * * 5",
-				},
-			},
-			now:              *deltaTimeAfterTopOfTheHour(6 * time.Hour),
-			expectedDuration: 20*time.Hour + nextScheduleDelta,
-		},
-	}
-	for _, tt := range tests {
-		t.Run(tt.name, func(t *testing.T) {
-			sched, err := cron.ParseStandard(tt.cj.Spec.Schedule)
-			if err != nil {
-				t.Errorf("error setting up the test, %s", err)
-			}
-			gotScheduleTimeDuration := nextScheduleTimeDuration(tt.cj, tt.now, sched)
-			if *gotScheduleTimeDuration < 0 {
-				t.Errorf("scheduleTimeDuration should never be less than 0, got %s", gotScheduleTimeDuration)
-			}
-			if !reflect.DeepEqual(gotScheduleTimeDuration, &tt.expectedDuration) {
-				t.Errorf("scheduleTimeDuration - got %s, want %s", gotScheduleTimeDuration, tt.expectedDuration)
+			if gotNumberOfMisses != tt.expectedNumberOfMisses {
+				t.Errorf("expectedNumberOfMisses - got %v, want %v", gotNumberOfMisses, tt.expectedNumberOfMisses)
 			}
 		})
 	}
